# -*- coding: utf-8 -*-
import django.contrib.auth.admin as auth_admin
import django.contrib.auth.models as auth_models
import django.db as db
import django.forms as forms
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages

import selectable.forms as select_forms
import basket

import lookups as c_lookups
import contacts.admin_forms as admin_forms
import contacts.admin_views as admv
import contacts.forms as contacts_forms
import contacts.models as contacts_models

from contacts import settings_messages


#### Helper function
def get_readonly_field(model_admin):
    "parse model_admin.fieldsets to set up a readonly mask"
    rof = []
    if model_admin.fieldsets:
        for label, opt in model_admin.fieldsets:
            for field in opt['fields']:
                if isinstance(field, (tuple, list)):
                    rof.extend(field)
                else:
                    rof.append(field)
    elif model_admin.fields:
        rof.extend(model_admin.fields)
    return rof


def debug_admin(model_admin, request, queryset):
#    import pdb; pdb.set_trace()
    return


class TechnicienFilter(admin.SimpleListFilter):
    title = 'technicien'
    parameter_name = 'technicien'

    def lookups(self, request, models):
        list_tech = (
            contacts_models.AdhLogin.objects
            .annotate(group_count=db.models.Count('groups'))
#            .filter(groups__name__icontains='gabnor')
            .filter(is_active=True, group_count__gte=1)
            .order_by('username')
            )
        return [(t.id, t.username) for t in list_tech]

    def queryset(self, request, queryset):
        if self.value():
            return queryset.filter(technicien__id__exact=self.value())
        else:
            return queryset
            
            
### Admin des utilisateurs
class AdminUser(auth_admin.UserAdmin):
    """
    On redéfinit l'admin pour les utilisateurs pour éviter l'édition de
    champs
    inutiles. On hérite de auth_admin.UserAdmin qui définit les urls d'accès
    aux formulaire de changement de mot de passe, etc.
    """
    ordering = ('username',)
    list_display = ('username', 'first_name', 'last_name', 'is_active')
    list_display_links = ('username',)
    list_filter = ('is_active',)

    fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (_('Personal info'), {'fields': ('first_name', 'last_name', 'email')}),
        (_('Permissions'), {'fields': ('is_active', 'is_superuser')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
        (_('Groups'), {'fields': ('groups',)}),
    )
    readonly_fields = ('last_login', 'date_joined')

    def save_model(self, request, obj, form, change):
        """
        Redéfinition de save_model pour forcer tous les utilisateurs à faire
        partie du staff (pour accéder à la partie admin) et superutilisateur
        (pour pouvoir tout modifier) puisqu'on n'utilise pas de droit.
        """
        # on force aussi le username en minuscules pour éviter les ambiguités
        obj.username = obj.username.lower()
        obj.is_superuser = False
        obj.is_staff = True
        obj.save()


class InlineStructure(admin.TabularInline):
    model = contacts_models.Structure
    extra = 0
    readonly_fields = ('ville', 'type_pm', 'f_juridique', 'statut',)
    fields = readonly_fields
    can_delete = False
    can_add = False
    verbose_name = u"Personne Morale liée"
    verbose_name_plural = u"Personnes Morales liées"
    fk_name = 'adherent'

    def struct_detail(self, obj):
        if obj.id is None:
            return u"-"
        return u'<a href="{0}">{1}</a>'.format(
            obj.get_absolute_url(), unicode(obj))
    struct_detail.short_description = u"Désignation"
    struct_detail.allow_tags = True


class AdminAdh(auth_admin.UserAdmin):
    """
    On redéfinit l'admin pour les utilisateurs pour éviter l'édition de
    champs
    inutiles. On hérite de auth_admin.UserAdmin qui définit les urls d'accès
    aux formulaire de changement de mot de passe, etc.
    """
    ordering = ('username',)
    list_display = ('username', 'is_active',)
    list_display_links = ('username',)
    list_filter = ('is_active',)

    fieldsets = (
        (None, {'fields': ('username', 'is_active', 'password')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    readonly_fields = ('last_login', 'date_joined')
    inlines = (InlineStructure,)

    def save_model(self, request, obj, form, change):
        """
        Redéfinition de save_model pour forcer tous les utilisateurs à faire
        partie du staff (pour accéder à la partie admin)
        """
        # on force aussi le username en minuscules pour éviter les ambiguités
        obj.username = obj.username.lower()
        obj.is_superuser = False
        obj.is_staff = True
        obj.save()


### Inline
# InlinePersonne doit être définie avant BaseAdminStructure
class InlinePersonne(admin.StackedInline):
    """
    Classe de personnalisation de l'admin pour le modèle Personne en inline.

    Note: on ne peut pas hériter de MixinAdminFiltrable à ce niveau à cause
    car le fonctionnement n'est pas le même.
    """
    form = admin_forms.PersonneAdminForm
    model = contacts_models.Personne
    readonly_fields = ('date_creation', 'date_derniere_modification',
                       'utilisateur_derniere_modification')
    fieldsets = (
            ('Désignation', {
                'fields': (('civilite', 'nom', 'prenom', 'fonction',), 'structure',
                           ('date_naissance', 'date_installation',),
                           'reseaux_engagements'),
                'classes': ('collapse',),
                }),
            ('Coordonnées', {
                'fields': ('adresse',
                           'commune', ('code_postal', 'ville'),
                           'courrier_structure',
                           'tel_fixe', 'tel_mobile', 'fax', 'courriel',
                           'pas_de_courriel'),
                'classes': ('collapse',),
                }),
            ('Enregistrement', {
                'fields': (('date_creation',
                           'date_derniere_modification',
                           'utilisateur_derniere_modification',),),
                'classes': ('collapse',),
                }),
            )
    extra = 0


class InlinePersonneRo(InlinePersonne):
    readonly_fields = get_readonly_field(InlinePersonne)
    can_delete = False
    can_add = False


class FieldsProjetBio(object):
    radio_fields = {
        "pond_savoirfaire": admin.HORIZONTAL,
        "pond_motivation": admin.HORIZONTAL,
        "pond_entourage": admin.HORIZONTAL,
        "pond_foncier": admin.HORIZONTAL,
        "pond_materiel": admin.HORIZONTAL,
        "pond_reglementation": admin.HORIZONTAL,
        "pond_commercialisation": admin.HORIZONTAL,
        "pond_financement": admin.HORIZONTAL,
        "pond_maindoeuvre": admin.HORIZONTAL,
        }

    @classmethod
    def get_fieldsets(cls, collapse=False):
        return (
            ('Projet', {
                    'fields': (
                        'agriculteur', 'technicien',
                        'description', 'freins_identifies',
                        'pond_savoirfaire', 'pond_motivation',
                        'pond_entourage', 'pond_foncier',
                        'pond_materiel', 'pond_reglementation',
                        'pond_commercialisation', 'pond_financement',
                        'pond_maindoeuvre',
                        'suites_a_donner',
                        'remarques',
                        ('date_probable_certification', 'date_certification'),
                        ('engagement_surf', 'engagement_oc_enjeu_eau'),
                        ('installation_bio', 'date_engagement_oc'),
                        ),
                    'classes': ('collapse', 'wide',)
                    if collapse else ('wide',),
                    }),
            )


class InlineProjetBio(admin.StackedInline):
    """
    Inline pour affichage des projets bio dans le formulaire d'un Agriculteur.
    """
    model = contacts_models.ProjetBio
    extra = 0
    radio_fields = FieldsProjetBio.radio_fields
    fieldsets = FieldsProjetBio.get_fieldsets(collapse=True)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        """
        On exclut l'utilisateur « admin » de la liste des techniciens.
        """
        if db_field.name == "technicien":
            kwargs["queryset"] = auth_models.User.objects.exclude(
                username='admin')
        return super(InlineProjetBio, self).formfield_for_foreignkey(
            db_field, request, **kwargs)


class InlineProjetBioRo(InlineProjetBio):
    readonly_fields = get_readonly_field(InlineProjetBio)
    can_delete = False
    can_add = False


class InlineAdhesionGabnor(admin.TabularInline):
    ""
    model = contacts_models.AdhesionGabnor
    verbose_name = "adhésion GABnor"
    verbose_name_plural = "adhésions GABnor"
    extra = 0
    fieldsets = (
            (None, {
                'fields': (('date', 'montant',),),
                }),
            )


class InlineAdhesionGabnorRo(InlineAdhesionGabnor):
    readonly_fields = get_readonly_field(InlineAdhesionGabnor)
    can_delete = False
    can_add = False


class InlineRdv(admin.TabularInline):
    model = contacts_models.Rdv
    extra = 0
    form = admin_forms.RdvForm
    verbose_name = "Rendez vous"
    verbose_name_plural = "Rendez vous"
    fields = ('date', 'technicien', 'type_rdv', 'projet_bio', 'titre_evt',
              'lieu', 'finance_prog', 'est_confirme',
              )

    def get_formset(self, *args, **kwargs):
        return super(InlineRdv, self).get_formset(*args, **kwargs)


class InlineRdvRo(InlineRdv):
    readonly_fields = get_readonly_field(InlineRdv)
    can_delete = False
    can_add = False


class RdvPersonneForInlineForm(forms.ModelForm):
    class Meta:
        model = contacts_models.RdvPersonne
        widgets = {
            'personne': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.PersonneLookup)
            }

class InlineRdvPersonne(admin.TabularInline):
    model = contacts_models.RdvPersonne
    extra = 0
    form = RdvPersonneForInlineForm
    verbose_name = "Participation personne"
    verbose_name_plural = "Participations des personnes"
    readonly_fields = ['is_organisateur']
    fields = ('rdv', 'personne', 'is_organisateur')


class InlineRdvPersonneRo(InlineRdvPersonne):
    readonly_fields = get_readonly_field(InlineRdvPersonne)
    can_delete = False
    can_add = False


class RdvPersonneMoraleForInlineForm(forms.ModelForm):
    class Meta:
        model = contacts_models.RdvPersonneMorale
        widgets = {
#            'structure': select_forms.AutoComboboxSelectWidget(
#                lookup_class=c_lookups.StructureLookup),
            'structure': select_forms.AutoCompleteSelectWidget(
                lookup_class=c_lookups.StructureLookup),
            }


class InlineRdvPersonneMorale(admin.StackedInline):
    model = contacts_models.RdvPersonneMorale
    extra = 0
    verbose_name = u"Participation personne morale "
    verbose_name_plural = u"Participations des personnes morales"
    fields = ('rdv', 'rdv_detail', 'structure', )
    readonly_fields = ('rdv_detail',)
    form = RdvPersonneMoraleForInlineForm
    # Ce champs doit etre defini dans la classe qui herite _directement_
    # de Stacked/Tabular Inline a cause d'une sombre histoire de metaclass
    def rdv_detail(self, obj):
        if obj.id is None or obj.rdv.id is None:
            return u"-"
        return u"<a href=\"{0}\">{1}</a>".format(
            obj.rdv.get_absolute_url(), obj.rdv)
    rdv_detail.short_description = u"Détail rdv"
    rdv_detail.allow_tags = True


class InlineRdvPersonneMoraleRo(InlineRdvPersonneMorale):
    readonly_fields = get_readonly_field(InlineRdvPersonneMorale)
    can_delete = False
    can_add = False


class InlineRdvPersonneMoraleStruct(InlineRdvPersonneMorale):
    verbose_name = u"Participation aux rdv collectifs"
    verbose_name_plural = u"Participations aux rdv collectifs"
    can_add = False
    fields = ('rdv_detail', 'structure',)

class InlineRdvPersonneMoraleStructRo(InlineRdvPersonneMoraleStruct):
    readonly_fields = get_readonly_field(InlineRdvPersonneMorale)
    can_delete = False
    can_add = False


class InlineAssolement(admin.TabularInline):
    """
    Classe d'admin pour les données d'assolement de l'observatoire.
    """
    model = contacts_models.Assolement
    form = admin_forms.FormAssolement
    verbose_name = "Assolement"
    verbose_name_plural = "Assolements"
    extra = 0
    fieldsets = (
            (None, {
                'fields': (
                   ('culture', 'detail_culture'),
                   ('surface_bio',
                    'surface_c1', 'surface_c2', 'surface_c3',
                    'surface_conv'),
                   ('tonnage', 'prix', 'client', 'type_circuit'),
                   ),
                }),
            )
    formfield_overrides = {
        db.models.DecimalField: {'widget': forms.TextInput(attrs={'size': 5})},
        db.models.FloatField: {'widget': forms.TextInput(attrs={'size': 5})},
        }


class InlineAssolementRo(InlineAssolement):
    readonly_fields = get_readonly_field(InlineAssolement)
    form = admin_forms.FormAssolementRo
    can_delete = False
    can_add = False


class InlineCommercialisation(admin.TabularInline):
    """
    Classe d'admin pour les données de commercialisation de l'observatoire.
    """
    model = contacts_models.Commercialisation
    verbose_name = "Commercialisation"
    verbose_name_plural = "Commercialisations"
    extra = 0
    fieldsets = (
            (None, {
                'fields': (
                   'type_produit',
                   'type_circuit',
                   'ch_aff',
                   'detail',
                   'bio',
                   ),
                }),
            )
    formfield_overrides = {
        db.models.TextField: {'widget': forms.Textarea(attrs={'rows': 2})},
        }


class InlineCommercialisationRo(InlineCommercialisation):
    readonly_fields = get_readonly_field(InlineCommercialisation)
    can_delete = False
    can_add = False


class InlineTransformation(admin.TabularInline):
    """
    Classe d'admin pour les données de transformation de l'observatoire.
    """
    model = contacts_models.Transformation
    verbose_name = "Transformation"
    verbose_name_plural = "Transformations"
    extra = 0
    formfield_overrides = {
        db.models.TextField: {'widget': forms.Textarea(attrs={'rows': 2})},
        }
    fieldsets = (
            (None, {
                'fields': (
                   ('ferme', 'type_pduit'),
                   'agrmt',
                   'list_pduit',
                   ('part_vol_glob', 'est_bio'),
                    'lait_abattoirs_vl',
                   ),
                }),
            )


class InlineTransformationRo(InlineTransformation):
    readonly_fields = get_readonly_field(InlineTransformation)
    can_delete = False
    can_add = False


class InlineDocumentJoint(admin.TabularInline):
    fields = ('date_depot', 'document',)
    readonly_fields = ('date_depot',)
    model = contacts_models.DocumentJoint
    extra = 0


class InlineDocumentJointRo(InlineDocumentJoint):
    def doc_detail(self, obj):
        if obj.id is None or obj.rdv.id is None:
            return u"-"
        return u"<a href=\"{0}\">{1}</a>".format(
            obj.document.url, obj.document.name)
    doc_detail.short_description = u"Document"
    doc_detail.allow_tags = True

    fields = ('date_depot', 'doc_detail',)
    readonly_fields = fields
    can_delete = False
    can_add = False


### ModelAdmin
# Admin de Personne
class AdminPersonne(basket.MixinBasketAdmin, admin.ModelAdmin):
    """
    Classe de personnalisation de l'admin pour le modèle Personne.
    """
    list_display = ('nom', 'prenom', 'structure',)
    list_display_links = ('nom', 'prenom', 'structure')
    list_filter = ('structure__type_pm', 'structure__statut')
    actions = (
        admv.gen_etiquettes, admv.gen_mailto, admv.gen_csv_publipostage,
        admv.gen_csv_complet,)
    basket_actions = (
        admv.gen_etiquettes, admv.gen_mailto, admv.gen_csv_publipostage,
        admv.gen_csv_complet,)
    search_fields = ('nom', 'prenom',)

    def save_model(self, request, obj, form, change):
        """
        Redéfinition de save_model pour assigner automatiquement le nom de
        l'utilisateur ayant effectué la modification.
        """
        obj.utilisateur_derniere_modification = request.user
        obj.save()

    inlines = [InlineRdvPersonne]


class AdminPersonneRo(AdminPersonne):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminPersonne)

    inlines = (InlineRdvPersonneRo,)

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminPersonneRo, self).change_view(
            request, object_id, extra_context)


class AdminStructure(
    basket.MixinBigFilter, basket.MixinBasketAdmin, admin.ModelAdmin):
    "Classe de personnalisation de l'admin pour le modèle Structure."
    class Media:
        css = {
            "all": ("css/contacts.css",),
        }
        js = ['js/list_filter_collapse.js']
# django.jQuery(
# "div#rdvpersonnemorale_set-group > div[id!='rdvpersonnemorale_set-empty']")
# .toggle()

    # change_list
    list_display = (
        'id', 'nom_affichage', 'statut', 'f_juridique', 'commune', 'type',
        'get_date_premiere_conversion')
    list_display_links = ('nom_affichage',)
    search_fields = ('designation', 'personne__nom', 'commune__nom',)
    list_filter = (
        'type_pm',
        'statut',
        'f_juridique',
        'type_activite',
        'type_activite_pri',
        'commune__zone',
        'commune__petite_region',
        'commune__arrondissement',
        )
    actions = (
        admv.gen_etiquettes, admv.gen_mailto, admv.gen_mailto_sympa,
        admv.gen_csv_publipostage, admv.gen_csv_complet,)
    basket_actions = (
        admv.gen_etiquettes, admv.gen_mailto, admv.gen_mailto_sympa,
        admv.gen_csv_publipostage, admv.gen_csv_complet,)
    # change_form
    form = admin_forms.AgriculteurAdminForm
    radio_fields = {"visibilite": admin.HORIZONTAL}
    date_hierarchie = 'date_creation'
    filter_horizontal = ('etiquettes', 'qual_compl', 'qual_compl_conv')
    readonly_fields = ('date_creation', 'date_derniere_modification',
                       'utilisateur_derniere_modification',
                       'premiere_adhesion', 'renouvellement_adhesion',
                       'adherent_edit')
    fieldsets = (
        ('Désignation',
         {'fields':
              (('designation', 'type_pm'),
               ('adherent', 'adherent_edit'),
               ('siret', 'f_juridique', 'statut'),
               'commentaire', 'visibilite', 'etiquettes'),
          }),
        ('Coordonnées', {
                'fields': ('adresse',
                           'commune', ('code_postal', 'ville'),
                           'tel_fixe', 'tel_mobile', 'fax', 'courriel',
                           'site_web'),
                'classes': ('collapse',),
                }),
        ("Informations sur l'exploitation",
         {'fields': (('type_activite', 'type_activite_pri'), 'qual_compl',
                     'qual_compl_conv',
                     ('sau_eau', 'sau_eau_bio')),
          'classes': ('collapse',),
          }),
        ('Infos adhésions Gabnor',
         {'fields': (('premiere_adhesion',
                     'renouvellement_adhesion',),),
          'classes': ('collapse',),
          }),
        ('Contact', {
                'fields': ('origine_contact',
                           ('annee_contact', 'mois_contact'),
                           ('date_creation', 'date_derniere_modification',
                            'utilisateur_derniere_modification'), ),
                'classes': ('collapse',),
                }),
        )
    inlines = (
        InlineAdhesionGabnor,
        InlinePersonne,
        InlineProjetBio,
        )
    classe_form_filtre = contacts_forms.FormFiltreStructure
    class_form_big_filter = contacts_forms.FormFiltreStructure

    def adherent_edit(self, obj):
        if obj.id is None or obj.adherent is None:
            return u"-"
        return u"<a href=\"{0}\">{1}</a>".format(
            reverse("admin:contacts_adhlogin_change", args=[obj.adherent.id]),
            unicode(obj.adherent))
    adherent_edit.short_description = u"Détail compte adhérent"
    adherent_edit.allow_tags = True

    def get_formsets(self, request, obj=None):
        for inline in self.get_inline_instances(request):
            resp = inline.get_formset(request, obj)
            if resp.model == contacts_models.Rdv:
                pass
            yield resp

    def nom_affichage(self, obj):
        return str(obj)
    nom_affichage.short_description = "Structure ou personne"

    def save_model(self, request, obj, form, change):
        """
        Redéfinition de save_model pour assigner automatiquement le nom de
        l'utilisateur ayant effectué la modification.
        """
        obj.utilisateur_derniere_modification = request.user
        obj.save()

    def save_formset(self, request, form, formset, change):
        """
        Assignation de l'utilisateur ayant effectué la dernière modification
        des objets Personne en Inline (pour lesquel il n'y a pas de méthode
        save_model).
        """
        instances = formset.save(commit=False)
        for instance in instances:
            instance.utilisateur_derniere_modification = request.user
            instance.save()
        formset.save_m2m()

        structure = form.instance

        # On vérifie d'abord qu'on se trouve dans le bon formset pour éviter de dupliquer des messages inutilement
        if type(formset).__name__ == "PersonneFormFormSet":
            if (structure.type_pm >= 1 and structure.personne_set.count() == 0):
                messages.add_message(request, messages.WARNING, settings_messages.AVERTISSEMENT_MESSAGE["message_personne_manquante"])

        elif type(formset).__name__ == "ProjetBioFormFormSet":
            if structure.type_pm == 2 and structure.projetbio_set.count() == 0 :
                messages.add_message(request, messages.WARNING, settings_messages.AVERTISSEMENT_MESSAGE["message_projet_bio_manquant"])
            elif structure.type_pm == 2 and structure.projetbio_set.count() != 0:
                # On vérifie qu'il y a au moins une date d'engagement OC
                test_date = True
                for projet in structure.projetbio_set.all():
                    if projet.date_engagement_oc != None:
                        test_date = False
                if test_date == True:
                    messages.add_message(request, messages.WARNING, settings_messages.AVERTISSEMENT_MESSAGE["message_projet_bio_manquant"])



class AdminStructureRo(AdminStructure):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminStructure)

    inlines = (
        InlineAdhesionGabnorRo,
        InlinePersonneRo,
        InlineProjetBioRo,
        )

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminStructureRo, self).change_view(
            request, object_id, extra_context)


### Admin de ProjetBio et dépendances
class AdminProjetBio(basket.MixinBasketAdmin, admin.ModelAdmin):
    """
    Classe d'admin pour afficher les projets bio de façon indépendante.
    """
    list_display = ('__unicode__', 'agri_id')
    list_display_links = ('__unicode__', )
    inlines = (InlineRdv,)
    search_fields = ('agriculteur__designation',
                     'agriculteur__personne__nom',
                     'agriculteur__commune__nom')
    list_filter = (TechnicienFilter,)
    actions = (admv.gen_csv_complet,)
    basket_actions = (admv.gen_csv_complet,)

    radio_fields = FieldsProjetBio.radio_fields
    fieldsets = FieldsProjetBio.get_fieldsets(collapse=False)

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "technicien":
            kwargs["queryset"] = auth_models.User.objects.exclude(
                username='admin')
        return super(AdminProjetBio, self).formfield_for_foreignkey(
            db_field, request, **kwargs)


class AdminProjetBioRo(AdminProjetBio):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminProjetBio)

    inlines = (InlineRdvRo,)

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminProjetBioRo, self).change_view(
            request, object_id, extra_context)


class AdminDonneesBio(basket.MixinBasketAdmin, admin.ModelAdmin):
    """
    Classe d'admin pour les données de l'observatoire (également éditées
    dans les formulaires des agriculteurs bio).
    """
    inlines = (InlineAssolement, InlineCommercialisation, InlineTransformation)
    search_fields = ('agriculteur_bio__designation',
                     'agriculteur_bio__personne__nom',
                     'agriculteur_bio__commune__nom')
    list_filter = ('annee', )
    list_display = ('__unicode__', 'agri_id')
#    date_hierarchy = 'date_collecte'
    actions = (admv.gen_csv_complet, admv.gen_csv_assolement, 
            admv.gen_csv_commercialisation, admv.gen_csv_transformation)
    basket_actions = (admv.gen_csv_complet,)
    # Ne fonctionne pas correctement -> à voir
    formfield_overrides = {
            db.models.BooleanField: {'widget': forms.CheckboxInput},
        }
    readonly_fields = ('ass_total', 'ass_total_bio',
                       'com_ca_total', 'com_pct_ca_bio',
                       'agri_detail',)
    fieldsets = (
            (None, {
                'fields': (
                   ('agriculteur_bio', 'agri_detail',),
                   ('date_collecte', 'technicien'),
                   'annee',
                   'organisme_certification',
                   'biodynamie',
                   'remarques',
                   ),
                }),
            (u"Aides", {
                'fields': (
                   ('aide_conversion_ab', 'date_aide_conversion_ab'),
                   ('aide_maintien_ab', 'aide_credit_impot',
                    'aide_certification_bio'),
                   ('aide_investissement', 'aide_installation',
                    'aide_mae_t'),
                   'aide_autres',
                   ('aide_accompagnement', 'aide_accompagnement_precisions'),
                   ),
                'classes': ('wide', 'collapse',),
                }),
            (u"Main d'œuvre", {
                'fields': (
                   ('etp_associes', 'etp_familiaux'),
                   ('etp_salaries_permanents', 'etp_saisonniers'),
                   ('etp_encadrants', 'personnes_insertion'),
                   ),
                'classes': ('wide', 'collapse',),
                }),
            (u"Transmission", {
                'fields': (
                   ('tr_date_fin_prevue', 'tr_repreneur_id'),
                   'tr_precisions',
                   ),
                'classes': ('wide', 'collapse',),
                }),
            (u"Commercialisation", {
                'fields': (
                   ('statut_commercialisation', 'statut_choisi'),
                   ('com_ca_total', 'com_pct_ca_bio'),
                   ),
                'classes': ('wide', 'collapse',),
                }),
            (u'Accueil', {
                'fields': (('acc_act_tour', 'acc_act_peda'),
                           ('acc_rest_bio', 'acc_act_autre'),
                           'detail_activites', ),
                'classes': ('wide', 'collapse',),
                }),
            (u"Lait de vaches", {
                'fields': (
                    # lait
                    ('lait_vaches', 'lait_nb_vaches', 'lait_vaches_conversion'),
                    ('lait_quota_laiterie', 'lait_quota_vente_directe'),
                    ('lait_prod_laiterie', 'lait_prod_vente_directe'),
                    'lait_laiterie',
                    # viande issue de vaches laitières
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Lait de chèvres", {
                'fields': (
                    ('lait_chevres', 'lait_nb_chevres',
                     'lait_chevres_conversion'),
                    'lait_chevres_prod',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Viande bovine", {
                'fields': (
                    ('bovine', 'bovine_nb_va', 'bovine_nb_conversion'),
                    ('bovine_reformes_cl', 'bovine_taurillons_cl'),
                    ('bovine_adultes_cc', 'bovine_veaux_cc'),
                    ('lait_reformes_cl', 'lait_taurillons_cl'),
                    ('lait_adultes_cc', 'lait_veaux_cc'),
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Porcs", {
                'fields': (
                    ('porcs', 'porcs_nb_truies'),
                    'porcs_nb_com',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Moutons", {
                'fields': (
                    ('moutons', 'moutons_nb_brebis' ),
                    'moutons_nb_agn_com',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Petits animaux", {
                'fields': (
                    ('animaux_lapins', 'animaux_volailles'),
                    'animaux_volailles_chair',
                    'animaux_nb_poulets',
                    'animaux_nb_lapins',
                    'animaux_nb_autres_volailles',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Autres productions animales", {
                'fields': (
                    'animaux_autres_prod',
                    'animaux_details_autres_prod',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Œufs", {
                'fields': (
                    'oeufs_production_bio',
                    'oeufs_nb_pondeuses', 'oeufs_nb_oeufs',
                    'oeufs_acheteur_cl',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Miel", {
                'fields': (
                    'miel_production_bio',
                    'miel_nb_ruches',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Formation", {
                'fields': (
                    ('formation_annee', 'formation_annee_laquelle'),
                    'formation_annee_commentaire',
                    ('formation_avenir', 'formation_avenir_type'),
                    'formation_avenir_details',
                   ),
                'classes': ('wide', 'collapse'),
                }),
            (u"Surface d'assolement", {
                'fields': (
                    ('ass_total', 'ass_total_bio',),
                   ),
                'classes': ('wide',),
                }),
        )


class AdminDonneesBioRo(AdminDonneesBio):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminDonneesBio)

    inlines = (InlineAssolementRo, InlineCommercialisationRo,
               InlineTransformationRo)

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminDonneesBioRo, self).change_view(
            request, object_id, extra_context)


### Admin de AdhesionGabnor
class AdminAdhesionGabnor(basket.MixinBasketAdmin, admin.ModelAdmin):
    """
    Classe d'admin pour les adhésions Gabnor (également éditées
    dans les formulaires des agriculteurs bio).
    """
    date_hierarchy = 'date'
    list_filter = ('date',)
    search_fields = ('agriculteur_bio__designation',
                     'agriculteur_bio__personne__nom',
                     'agriculteur_bio__commune__nom')
    actions = (admv.gen_csv_complet,)
    basket_actions = (admv.gen_csv_complet,)
    fields = ('agriculteur_bio', 'date', 'montant', 'annee')


class AdminAdhesionGabnorRo(AdminAdhesionGabnor):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminAdhesionGabnor)

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminAdhesionGabnorRo, self).change_view(
            request, object_id, extra_context)


class AdminRdv(basket.MixinBasketAdmin, admin.ModelAdmin):
    search_fields = (
        'titre_evt__designation',
        'lieu__nom',
        'personne_morale__designation',
        'personne_morale__personne__nom',
        'rdvpersonnemorale__structure__designation',
        'rdvpersonnemorale__structure__personne__nom'
        )
    list_filter = ('type_rdv', 'projet_bio__technicien', TechnicienFilter)
    list_display = ('date', 'type_rdv', 'technicien', 'personne_morale',
                    'titre_evt',)
    date_hierarchy = 'date'
    basket_actions = (admv.gen_csv_complet,)
    actions = (admv.gen_csv_complet,)

    readonly_fields = ('struct_af_org', 'struct_af_int',)
    fieldsets = (
        (None, {'fields': (
                    ('date',
                    'type_rdv',),
                    ('technicien',
                    'lieu',),
                    'commentaire',
                    (
                    'finance_prog',
                    'acc_duree',
                    'acc_distance',),
                    ),}),
        (u"Rendez vous individuels", {'fields': (
                    ('personne_morale',
                    'projet_bio',),
                    ),}),
        (u"Évenements collectifs", {'fields': (
                    ('est_confirme',
                    'titre_evt',),
                    ('zone_cible', 'public_cible',),
                    ('organisateur', 'struct_af_org',),
                    ('intervenant', 'struct_af_int'),
                    ('objectif', 'res_attendu',)
                    ),}),
        )
    form = admin_forms.RdvForm
    inlines = (
        InlineRdvPersonne, InlineDocumentJoint)
    
    def lookup_allowed(self, key, value):
        if key in ('personnes__structure'):
            return True
        return super(AdminRdv, self).lookup_allowed(key, value)
    
    def save_formset(self, request, form, formset, change):
        formset.save()
        rdv = form.instance
        if type(formset).__name__ == "RdvPersonneFormFormSet":
            ## En attendant une eventuelle migration
            #pers_suppr = []
            #for row in formset:
                #if row.cleaned_data['DELETE'] == True:
                    #pers_suppr.append(row.cleaned_data['personne'])
            #for structure in rdv.personnes_morales.all():
                #pers = structure.personne_set.all()[:1]
                #if (len(pers)==1 and (pers[0] not in rdv.personnes.all())
                    #and pers[0] not in pers_suppr):
                    #r = contacts_models.RdvPersonne(
                    #rdv=rdv, personne=pers[0])
                    #r.save()
                    
            #rdv.personnes_morales.clear()
            #for participant in rdv.personnes.all():
                #if participant.structure not in rdv.personnes_morales.all():
                    #r = contacts_models.RdvPersonneMorale(
                    #rdv=rdv, structure=participant.structure)
                    #r.save()
            if ('collect' in rdv.get_cat() and
            rdv.rdvpersonne_set.count() == 0 and
            rdv.est_confirme):
                messages.add_message(request, 
                messages.WARNING, 
                settings_messages.AVERTISSEMENT_MESSAGE["rdv_collec_sans_part"])


class AdminRdvRo(AdminRdv):
    def get_readonly_fields(self, request, obj):
        return get_readonly_field(AdminRdv)

    inlines = (
        InlineRdvPersonneRo, InlineDocumentJointRo)

    def change_view(self,  request, object_id, extra_context=None):
        "ensure no data is saved on a readonly ModelAdmin"
        request.method = 'GET'  # never consider a POST
        return super(AdminRdvRo, self).change_view(
            request, object_id, extra_context)


### Enregistrement des modèles à administrer
def register_models(site):
    """
    Enregistrement des modèles dans un AdminSite passé en paramètre.
    """
    # Modèle User avec admin personnalisée
    site.register(auth_models.User, AdminUser)
    site.register(auth_models.Group, auth_admin.GroupAdmin)
    # Modèles principaux indispensables
    site.register(contacts_models.Structure, AdminStructure)
    site.register(contacts_models.StructureRo, AdminStructureRo)
    site.register(contacts_models.Personne, AdminPersonne)
    site.register(contacts_models.PersonneRo, AdminPersonneRo)
    # Facultatif: administration indépendante de données qu'on retrouve dans en
    # inline dans les autres pages.
    site.register(contacts_models.ProjetBio, AdminProjetBio)
    site.register(contacts_models.ProjetBioRo, AdminProjetBioRo)
    site.register(contacts_models.DonneesBio, AdminDonneesBio)
    site.register(contacts_models.DonneesBioRo, AdminDonneesBioRo)
    site.register(contacts_models.AdhesionGabnor, AdminAdhesionGabnor)
    site.register(contacts_models.AdhesionGabnorRo, AdminAdhesionGabnorRo)
    site.register(contacts_models.Rdv, AdminRdv)
    site.register(contacts_models.RdvRo, AdminRdvRo)
    site.register(contacts_models.DocumentJoint)
    site.register(contacts_models.AdhLogin, AdminAdh)
# fait dans urls.py
#register_models(site)

# EOF
